@@ -75,11 +75,38 @@ class PixelMapFit:
     
     def __init__(self, targetpixelfile, gaia=True, magnitude_limit=18, 
                  frequencies=[], frequnit=u.uHz, principal_components = 5, 
-                 aperture=None):
+                 aperture=None, method = 'Gaussian', sigma=None,pca_only = False, **kwargs):
         
         self.tpf = targetpixelfile
+        self.method = method
         #Defining an aperture that will be used in plotting and making empty 2-d arrays of the correct size for masks
         self.aperture = aperture
+        self.pca_only = pca_only
+        if self.pca_only == True:
+            self.raw_lc1 = self.tpf.to_lightcurve(aperture_mask=self.aperture)
+            self.quality_mask = [np.isfinite(self.raw_lc1['flux']*self.raw_lc1['flux_err'])]
+            if principal_components !=None:
+                self.dm = lk.DesignMatrix(self.tpf.flux[:,~self.tpf.pipeline_mask][np.isfinite(self.raw_lc1['flux']*self.raw_lc1['flux_err']),:][np.where(self.raw_lc1[self.quality_mask[0]].quality ==0)[0],:], name='regressors').pca(principal_components)
+                self.raw_lc = self.raw_lc1[self.quality_mask[0]]
+                self.raw_lc = self.raw_lc[np.where(self.raw_lc1[self.quality_mask[0]].quality ==0)]
+
+                rc = lk.RegressionCorrector(self.raw_lc)
+                self.corrected_lc = rc.correct(self.dm.append_constant())
+                fig,ax = plt.subplots(1,2,figsize=(14,5),sharey=True)
+                ax[0].plot(self.tpf.time.value[self.quality_mask[0]][np.where(self.raw_lc1[self.quality_mask[0]].quality ==0)], self.dm.values*1 + np.arange(self.principal_components)*0.2)
+                ax[0].set_title(r'Principal Components Contributions')
+                ax[0].set_xlabel('Offset')
+                ax[1].plot(pgs[0].frequency.value,np.array([pgs[i].power.value*10+np.arange(self.principal_components)[i]*0.2 for i in range(len(pgs))]).T)
+                ax[1].set_xlim(self.frequency_list.min(),self.frequency_list.max())
+                for i in np.arange(0,len(self.frequency_list)):
+                    ax[1].axvline(x = self.frequency_list[i],color='r',linestyle='--',linewidth=1)
+                g2 = self.raw_lc.plot(label='Raw light curve')
+                self.corrected_lc.plot(ax=g2, label='Corrected light curve')
+                quit()
+                
+            else:
+                quit()
+            
         if self.aperture is None:
             self.aperture = targetpixelfile.pipeline_mask
             if targetpixelfile.pipeline_mask.any() == False:
@@ -88,12 +115,21 @@ class PixelMapFit:
     
         
         # Make a design matrix and pass it to a linear regression corrector
-        self.raw_lc = self.tpf.to_lightcurve(aperture_mask=self.aperture)
-        self.dm = lk.DesignMatrix(self.tpf.flux[:, ~self.aperture], name='regressors').pca(principal_components)
-        rc = lk.RegressionCorrector(self.raw_lc)
-        corrected_lc = rc.correct(self.dm.append_constant())
-        corrected_lc[np.where(corrected_lc.quality == 0)]
-        self.corrected_lc = corrected_lc.remove_outliers()
+        self.raw_lc1 = self.tpf.to_lightcurve(aperture_mask=self.aperture)
+        self.quality_mask = [np.isfinite(self.raw_lc1['flux']*self.raw_lc1['flux_err'])]
+        if principal_components !=None:
+            self.dm = lk.DesignMatrix(self.tpf.flux[:,~self.tpf.pipeline_mask][np.isfinite(self.raw_lc1['flux']*self.raw_lc1['flux_err']),:][np.where(self.raw_lc1[self.quality_mask[0]].quality ==0)[0],:], name='regressors').pca(principal_components)
+            self.raw_lc = self.raw_lc1[self.quality_mask[0]]
+            self.raw_lc = self.raw_lc[np.where(self.raw_lc1[self.quality_mask[0]].quality ==0)]
+        
+            rc = lk.RegressionCorrector(self.raw_lc)
+            self.corrected_lc = rc.correct(self.dm.append_constant())
+        else:
+            self.raw_lc = self.raw_lc1[self.quality_mask[0]]
+            self.raw_lc = self.raw_lc[np.where(self.raw_lc1[self.quality_mask[0]].quality ==0)]
+            self.corrected_lc = self.raw_lc
+            
+        #self.corrected_lc = corrected_lc.remove_outliers()
         self.frequency_list = np.asarray((frequencies*frequnit).to(1/u.d))
         self.principal_components = principal_components
         
@@ -170,7 +206,7 @@ class PixelMapFit:
 
             params = model.make_params()
 
-            result = model.fit(corrected_lc.flux.value,params,time=times,weights=corrected_lc.flux_err.value)
+            result = model.fit(corrected_lc.flux.value,params,time=times,weights=1/corrected_lc.flux_err.value)
             
             final_phases = [result.best_values['f{0:d}phase'.format(j)] for j in np.arange(len(frequency_list))]
 
@@ -210,7 +246,7 @@ class PixelMapFit:
 
             params = model.make_params()
 
-            result = model.fit(corrected_lc.flux.value,params,time=times,weights=corrected_lc.flux_err.value)
+            result = model.fit(corrected_lc.flux.value,params,time=times,weights=1/corrected_lc.flux_err.value)
             
             return result
         
@@ -232,8 +268,16 @@ class PixelMapFit:
                 
                 #Getting the light curve for a pixel and excluding any flagged data
                 lightcurve = self.tpf.to_lightcurve(aperture_mask=mask)
-                rcc = lk.RegressionCorrector(lightcurve)
-                lc = rcc.correct(self.dm.append_constant())
+                lightcurve = lightcurve[self.quality_mask[0]]
+                lightcurve = lightcurve[np.where(self.raw_lc1[self.quality_mask[0]].quality ==0)]
+
+
+                if principal_components !=None:
+                    rcc = lk.RegressionCorrector(lightcurve)
+                    lc = rcc.correct(self.dm.append_constant())
+                else:
+                    lc=lightcurve
+                #lc = lc.remove_outliers()
                               
                 bestfit = Obtain_Final_Fit(self.tpf,lc,self.frequency_list,self.final_phases)
                 heat = np.asarray([bestfit.best_values['f{0:d}amp'.format(n)] for n in np.arange(len(self.frequency_list))])
@@ -245,8 +289,8 @@ class PixelMapFit:
                     
         heats = np.asarray(heats)
         heats_error = np.asarray(heats_error)
-        self.heatmap = heats.T
-        self.heatmap_error = heats_error.T
+        self.heats = heats.T
+        self.heats_error = heats_error.T
         self.timeserieslength = (self.tpf.time.max()-self.tpf.time.min()).value
         self.gaiadata = None
         
@@ -303,55 +347,98 @@ class PixelMapFit:
             self.gaiadata = source
         class frequency_heatmap:
 
-            def __init__(self,tpf,heats,heats_error,frequencies,gaia_data):
+            def __init__(self,tpf,heats,heats_error,frequencies,gaia_data,method):
                 self.heat_stamp = heats
                 self.gaiadata=gaia_data
                 self.heatmap_error = heats_error
                 self.size = tpf.pipeline_mask.shape
                 self.frequencies= frequencies
                 self.tpf = tpf
+                self.method = method
             def location(self):
                 
-                #Residuals to minimize relative to the error bars
-                def residual(params, amp, amperr):
+                if self.method == 'PRF':
+                    
+                    self.prf = PRF.TESS_PRF(cam = self.tpf.camera, ccd = self.tpf.ccd,
+                                        sector = self.tpf.sector, 
+                                        colnum = self.tpf.column+self.size[0]/2.,
+                                        rownum = self.tpf.row+self.size[1]/2., 
+                                        **kwargs)
+                    #self.prf = PRF.Gaussian_PRF(sigma)
 
-                    x = params['x']
-                    y = params['y']
-                    #sigma = params['sigma']
-                    prf = PRF.TESS_PRF(cam = self.tpf.camera, ccd = self.tpf.ccd,
-                                           sector = self.tpf.sector, colnum = self.tpf.column,
-                                           rownum = self.tpf.row, localdatadir=None)
+                    #Residuals to minimize relative to the error bars
+                    def residual(params, amp, amperr, prf):
+
+                        x = params['x']
+                        y = params['y']
+
+                        res = []
+                        localprf = self.prf.locate(x+.5, y+.5, (self.size[0],self.size[1]))
+                        for i in np.arange(len(frequencies)):
+                            height = params['height{0:d}'.format(i)]
+                            model = height*localprf
+
+                            res.extend( [(amp[i].reshape(self.size)-model) / amperr[i].reshape(self.size)])
 
 
-                    res = []
-                    for i in np.arange(len(frequencies)):
-                        height = params['height{0:d}'.format(i)]
-                        #prf = PRF.Gaussian_PRF(sigma)
-                        model = height*prf.locate(x+.5, y+.5, (self.size[0],self.size[1]))
+                        return np.asarray(res)
 
-                        res.extend( [(amp[i].reshape(self.size)-model) / amperr[i].reshape(self.size)])
+                    #Set starting values to converge from
+                    self.heatmap_error[np.where(self.heatmap_error==None)]=np.nan
+                    self.heatmap_error[np.where(self.heatmap_error==0)]=np.nan
+                    #reshaping composite heatmap for user to plot
+                    composite_heatmap = self.heat_stamp.sum(axis=0).reshape(self.size) / ((np.nansum(self.heatmap_error**2,axis=0))**(1/2)).reshape(self.size)#issue with numpy using sqrt?
+                    c = np.where(composite_heatmap==composite_heatmap.max())
 
 
-                    return np.asarray(res)
+                    params = Parameters()
+                    for i in np.arange(len(frequencies)):#
+                        params.add('height{0:d}'.format(i), value=np.max(self.heat_stamp[i]))
+                    params.add('x', value=c[1][0])#c[0]) 
+                    params.add('y', value=c[0][0])#c[1])
+                    #params.add('sigma', value=1)
+                else:
+                    if sigma is None:
+                        self.sigma=0.8
+                    else:
+                        self.sigma = sigma
+                    self.prf = PRF.Gaussian_PRF(self.sigma)
 
-                #Set starting values to converge from
-                self.heatmap_error[np.where(self.heatmap_error==None)]=np.nan
-                self.heatmap_error[np.where(self.heatmap_error==0)]=np.nan
-                #reshaping composite heatmap for user to plot
-                composite_heatmap = self.heat_stamp.sum(axis=0).reshape(self.size) / ((np.nansum(self.heatmap_error**2,axis=0))**(1/2)).reshape(self.size)#issue with numpy using sqrt?
-                c = np.where(composite_heatmap==composite_heatmap.max())
+                    #Residuals to minimize relative to the error bars
+                    def residual(params, amp, amperr, prf):
+
+                        x = params['x']
+                        y = params['y']
+                        #sigma = params['sigma'] #KJB removed
+                        
+                        res = []
+                        for i in np.arange(len(frequencies)):
+                            height = params['height{0:d}'.format(i)]
+                            model = height*self.prf.locate(x+.5, y+.5, (self.size[0],self.size[1]))
+
+                            res.extend( [(amp[i].reshape(self.size)-model) / amperr[i].reshape(self.size)])
 
 
-                params = Parameters()
-                for i in np.arange(len(frequencies)):#
-                    params.add('height{0:d}'.format(i), value=np.max(self.heat_stamp[i]))
-                params.add('x', value=c[1][0])#c[0]) 
-                params.add('y', value=c[0][0])#c[1])
-                #params.add('sigma', value=1)
+                        return np.asarray(res)
 
-                
+                    #Set starting values to converge from
+                    self.heatmap_error[np.where(self.heatmap_error==None)]=np.nan
+                    self.heatmap_error[np.where(self.heatmap_error==0)]=np.nan
+                    #reshaping composite heatmap for user to plot
+                    composite_heatmap = self.heat_stamp.sum(axis=0).reshape(self.size) / ((np.nansum(self.heatmap_error**2,axis=0))**(1/2)).reshape(self.size)#issue with numpy using sqrt?
+                    c = np.where(composite_heatmap==composite_heatmap.max())
+
+
+                    params = Parameters()
+                    for i in np.arange(len(frequencies)):#
+                        params.add('height{0:d}'.format(i), value=np.max(self.heat_stamp[i]))
+                    params.add('x', value=c[1][0])#c[0]) 
+                    params.add('y', value=c[0][0])#c[1])
+                    #params.add('sigma', value=1)        # KJB: not fitted
+                    
+                    
                 #Do the fit
-                minner = Minimizer(residual, params, fcn_args=(self.heat_stamp, self.heatmap_error))
+                minner = Minimizer(residual, params, fcn_args=(self.heat_stamp, self.heatmap_error, self.prf))
                 self.result = minner.minimize()
                 fit = self.result.params.valuesdict()
                 self.x = fit['x']
@@ -378,10 +465,10 @@ class PixelMapFit:
                     starlist = pd.DataFrame.from_dict(stars)
                     self.stars = starlist.sort_values(by=[r'distance'])
                     
-        fh = frequency_heatmap(self.tpf,self.heatmap,self.heatmap_error,self.frequency_list,self.gaiadata) 
+        fh = frequency_heatmap(self.tpf,self.heats,self.heats_error,self.frequency_list,self.gaiadata,self.method) 
         fh.location()
         self.location = [fh.x,fh.y]
-        self.heatmap = self.heatmap.sum(axis=0).reshape(self.aperture.shape[0],self.aperture.shape[1]) / np.sqrt((self.heatmap_error**2).sum(axis=0)).reshape(self.aperture.shape[0],self.aperture.shape[1])
+        self.heatmap = self.heats.sum(axis=0).reshape(self.aperture.shape[0],self.aperture.shape[1]) / np.sqrt((self.heats_error**2).sum(axis=0)).reshape(self.aperture.shape[0],self.aperture.shape[1])
         self.result = fh.result
         if (self.gaiadata !=None):
             fh.star_list()
@@ -398,18 +485,24 @@ class PixelMapFit:
         plt.xlim(-.5,self.aperture.shape[1]-1+.5)
         plt.ylim(-.5,self.aperture.shape[0]-1+.5)
         report_fit(self.result)
-        if (np.asarray([self.result.params['height{0:d}'.format(j)].stderr for j in np.arange(len(self.frequency_list))]) / np.asarray([self.result.params['height{0:d}'.format(j)].value for j in np.arange(len(self.frequency_list))])).any() >.2:
-            Warning('Frequencies used may not all belong to the same source and provided fit could be unreliable')
+        if (np.asarray([self.result.params['height{0:d}'.format(j)].stderr for j in np.arange(len(self.frequency_list))]) / np.asarray([self.result.params['height{0:d}'.format(j)].value for j in np.arange(len(self.frequency_list))])>.2).any():
+            warnings.warn('Frequencies used may not all belong to the same source and provided fit could be unreliable')
     
     def pca(self):
-        plt.figure(figsize=(12,5))
-        plt.plot(self.tpf.time.value, self.dm.values + np.arange(self.principal_components)*0.2)
-        plt.title('Principal Components Contributions')
-        plt.xlabel('Offset')
-        g2 = self.raw_lc.plot(label='Raw light curve')
-        self.corrected_lc.plot(ax=g2, label='Corrected light curve')
-
-        plt.show()
+        if self.principal_components==None:
+            pass
+        else:
+            
+            fig,ax = plt.subplots(1,2,figsize=(14,5),sharey=True)
+            ax[0].plot(self.tpf.time.value[self.quality_mask[0]][np.where(self.raw_lc1[self.quality_mask[0]].quality ==0)], self.dm.values*1 + np.arange(self.principal_components)*0.2)
+            ax[0].set_title(r'Principal Components Contributions')
+            ax[0].set_xlabel('Offset')
+            ax[1].plot(pgs[0].frequency.value,np.array([pgs[i].power.value*10+np.arange(self.principal_components)[i]*0.2 for i in range(len(pgs))]).T)
+            ax[1].set_xlim(self.frequency_list.min(),self.frequency_list.max())
+            for i in np.arange(0,len(self.frequency_list)):
+                ax[1].axvline(x = self.frequency_list[i],color='r',linestyle='--',linewidth=1)
+            g2 = self.raw_lc.plot(label='Raw light curve')
+            self.corrected_lc.plot(ax=g2, label='Corrected light curve')
 
     def plot(self,save = None,figuresize = (10,10)):
         plt.figure(figsize = (figuresize))
